﻿using DataAccess.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataAccess.DAOs
{
    public class QuestionCategoryDAO
    {
        private static QuestionCategoryDAO instance = null;
        private static readonly object instanceLock = new object();
        private QuestionCategoryDAO() { }
        public static QuestionCategoryDAO Instance
        {
            get
            {
                lock (instanceLock)
                {
                    if (instance == null)
                        instance = new QuestionCategoryDAO();
                }
                return instance;
            }
        }

        public List<QuestionCategory> GetQuestionCategories(int? status)
        {
            var dbContext = new EosContext();
            return status == null ? dbContext.QuestionCategories.ToList() 
                : dbContext.QuestionCategories.Where(_ => _.Status == status).ToList();
        }

        public QuestionCategory GetQuestionCategory(int id, int? status)
        {
            var dbContext = new EosContext();
            return status ==  null ? dbContext.QuestionCategories.Find(id) 
                : dbContext.QuestionCategories.SingleOrDefault(_ => _.Status == status && _.CategoryId == id);
        }

        public QuestionCategory GetQuestionCategory(int? id, string name)
        {
            var dbContext = new EosContext();
            return id == null 
                ? dbContext.QuestionCategories.SingleOrDefault(_ => _.CategoryName.Equals(name))
                : dbContext.QuestionCategories.SingleOrDefault(_ => _.CategoryName.Equals(name) && _.CategoryId != id);
        }

        public void AddQuestionCategory(QuestionCategory questionCategory)
        {
            var dbContext = new EosContext();
            dbContext.QuestionCategories.Add(questionCategory);
            dbContext.SaveChanges();
        }

        public void UpdateQuestionCategory(QuestionCategory questionCategory)
        {
            var dbContext = new EosContext();
            dbContext.QuestionCategories.Update(questionCategory);
            dbContext.SaveChanges();
        }

        public void RemoveQuestionCategory(int id)
        {
            var dbContext = new EosContext();
            var questionCategory = dbContext.QuestionCategories.Find(id);
            questionCategory.Status = questionCategory.Status == 1 ? 0 : 1; ;
            dbContext.QuestionCategories.Update(questionCategory);
            dbContext.SaveChanges();
        }
    }
}
