﻿using DataAccess.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataAccess.DAOs
{
    public class QuestionDAO
    {
        private static QuestionDAO instance = null;
        private static readonly object instanceLock = new object();
        private QuestionDAO() { }
        public static QuestionDAO Instance
        {
            get
            {
                lock (instanceLock)
                {
                    if (instance == null)
                        instance = new QuestionDAO();
                }
                return instance;
            }
        }

        public List<Question> GetQuestions()
        {
            var dbContext = new EosContext();
            return dbContext.Questions.Include(_ => _.Category).Include(_ => _.Subject).ToList();
        }

        public Question GetQuestion(int id)
        {
            var dbContext = new EosContext();
            return dbContext.Questions.Include(_ => _.Category).Include(_ => _.Subject).SingleOrDefault(_ => _.QuestionId == id);
        }
        public Question GetQuestion(int? id, string content)
        {
            var dbContext = new EosContext();
            return id == null 
                ? dbContext.Questions.Include(_ => _.Category).Include(_ => _.Subject).SingleOrDefault(_ => _.QuestionId == id)
                : dbContext.Questions.Include(_ => _.Category).Include(_ => _.Subject).SingleOrDefault(_ => _.QuestionId == id && _.Content.Equals(content));
        }

        public void AddQuestion(Question question)
        {
            var dbContext = new EosContext();
            dbContext.Questions.Add(question);
            dbContext.SaveChanges();
        }

        public void UpdateQuestion(Question question)
        {
            var dbContext = new EosContext();
            dbContext.Questions.Update(question);
            dbContext.SaveChanges();
        }

        public void RemoveQuestion(int id)
        {
            var dbContext = new EosContext();
            var question = dbContext.Questions.Find(id);
            question.Status = question.Status == 1 ? 0 : 1;
            dbContext.Questions.Update(question);
            dbContext.SaveChanges();
        }
    }
}
