﻿using DataAccess.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataAccess.DAOs
{
    public class SubjectCategoryDAO
    {
        private static SubjectCategoryDAO instance = null;
        private static readonly object instanceLock = new object();
        private SubjectCategoryDAO() { }
        public static SubjectCategoryDAO Instance
        {
            get
            {
                lock (instanceLock)
                {
                    if (instance == null)
                        instance = new SubjectCategoryDAO();
                }
                return instance;
            }
        }

        public List<SubjectCategory> GetSubjectCategories(int? status)
        {
            var dbContext = new EosContext();
            return status == null ? dbContext.SubjectCategories.ToList() 
                : dbContext.SubjectCategories.Where(_ => _.Status == status).ToList();
        }

        public SubjectCategory GetSubjectCategory(int id, int? status)
        {
            var dbContext = new EosContext();
            return status == null ? dbContext.SubjectCategories.Find(id) 
                : dbContext.SubjectCategories.SingleOrDefault(_ => _.Status == status && _.CategoryId == id);
        }

        public SubjectCategory GetSubjectCategory(string name)
        {
            var dbContext = new EosContext();
            return dbContext.SubjectCategories.SingleOrDefault(_ => _.CategoryName.Equals(name));
        }

        public SubjectCategory GetSubjectCategory(int? id, string name)
        {
            var dbContext = new EosContext();
            return id == null
                ? dbContext.SubjectCategories.SingleOrDefault(_ => _.CategoryName.Equals(name))
                : dbContext.SubjectCategories.SingleOrDefault(_ => _.CategoryName.Equals(name) && _.CategoryId != id);
        }

        public void AddSubjectCategory(SubjectCategory subjectCategory)
        {
            var dbContext = new EosContext();
            dbContext.SubjectCategories.Add(subjectCategory);
            dbContext.SaveChanges();
        }

        public void UpdateSubjectCategory(SubjectCategory subjectCategory)
        {
            var dbContext = new EosContext();
            dbContext.SubjectCategories.Update(subjectCategory);
            dbContext.SaveChanges();
        }

        public void RemoveSubjectCategory(int id)
        {
            var dbContext = new EosContext();
            var subjectCategory = dbContext.SubjectCategories.Find(id);
            subjectCategory.Status = subjectCategory.Status == 1 ? 0 : 1; ;
            dbContext.SubjectCategories.Update(subjectCategory);
            dbContext.SaveChanges();
        }
    }
}
