﻿using DataAccess.DTOs;
using DataAccess.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DataAccess.DAOs
{
    public class SubjectDAO
    {
        private static SubjectDAO instance = null;
        private static readonly object instanceLock = new object();
        private SubjectDAO() { }
        public static SubjectDAO Instance
        {
            get
            {
                lock (instanceLock)
                {
                    if (instance == null)
                        instance = new SubjectDAO();
                }
                return instance;
            }
        }

        public List<Subject> GetSubjects(int? status)
        {
            var dbContext = new EosContext();
            return status == null ? dbContext.Subjects.Include(_ => _.Category).ToList()
                : dbContext.Subjects.Include(_ => _.Category).Where(_ => _.Status == status).ToList();
        }

        public Subject GetSubject(int id, int? status)
        {
            var dbContext = new EosContext();
            return status == null ? dbContext.Subjects.Include(_ => _.Category).SingleOrDefault(_ => _.SubjectId == id)
                : dbContext.Subjects.Include(_ => _.Category).SingleOrDefault(_ => _.Status == status && _.SubjectId == id);
        }

        public Subject GetSubject(string name)
        {
            var dbContext = new EosContext();
            return dbContext.Subjects.SingleOrDefault(_ => _.SubjectName.Equals(name));
        }

        public Subject GetSubject(int? id, string name)
        {
            var dbContext = new EosContext();
            return id == null
                ? dbContext.Subjects.SingleOrDefault(_ => _.SubjectName.Equals(name))
                : dbContext.Subjects.SingleOrDefault(_ => _.SubjectName.Equals(name) && _.CategoryId != id);
        }

        public void AddSubject(Subject subject)
        {
            var dbContext = new EosContext();
            dbContext.Subjects.Add(subject);
            dbContext.SaveChanges();
        }

        public void UpdateSubject(Subject subject)
        {
            var dbContext = new EosContext();
            dbContext.Subjects.Update(subject);
            dbContext.SaveChanges();
        }

        public void RemoveSubject(int id)
        {
            var dbContext = new EosContext();
            var subject = dbContext.Subjects.Find(id);
            subject.Status = subject.Status == 1 ? 0 : 1; ;
            dbContext.Subjects.Update(subject);
            dbContext.SaveChanges();
        }
    }
}
