using System.Net.Http;
using TMPro;
using UnityEngine;
using Newtonsoft.Json;
using System.Collections.Generic;
using Photon.Realtime;
using Photon.Pun;
using UnityEngine.Networking;

public class Login : MonoBehaviour, IConnectionCallbacks
{
    [SerializeField] private TextMeshProUGUI username;
    [SerializeField] private TextMeshProUGUI password;
    [SerializeField] private TextMeshProUGUI message;

    private string authToken = "";

    [System.Serializable]
    public class RespondMessage
    {
        public bool Success { get; set; }
        public string Message { get; set; }
        public string Data { get; set; }
    }

    public async void CallLoginAPI()
    {
        string baseURL = "http://localhost:5172";
        string endpoint = "/User/Login";
        string email = CleanInput(username.text.Trim());
        string pwd = CleanInput(password.text.Trim());

        using (HttpClient client = new HttpClient())
        {
            string apiUrl = $"{baseURL}{endpoint}?email={email}&pwd={pwd}";
            //Debug.Log(apiUrl.Trim().Equals("http://localhost:5172/User/Login?email=admin&pwd=admin"));

            using (HttpResponseMessage res = await client.GetAsync(apiUrl))
            {
                using (HttpContent content = res.Content)
                {
                    string data = await content.ReadAsStringAsync();
                    RespondMessage respondMessage = JsonConvert.DeserializeObject<RespondMessage>(data);
                    if (respondMessage.Success)
                    {
                        authToken = respondMessage.Data.ToString();
                        //AuthenticateWithToken("eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJlbWFpbCI6ImFkbWluIiwicm9sZSI6IjAiLCJUb2tlbklkIjoiYjcxOWIxYzEtNmI4MC00N2U1LThjMGQtYjU4ZTAyN2UyMmE4IiwibmJmIjoxNjkwNDUzMTc3LCJleHAiOjE2OTA0NTQ5NzcsImlhdCI6MTY5MDQ1MzE3N30.W-0PXJ9GAmwOOXlKi8cEr3lBkwx9rGbpLKCCUYJkyCA");
                        AuthenticateWithToken(authToken);

                        if (PhotonNetwork.IsConnected)
                        {
                            message.text = "Login successful!";
                        }
                        else
                        {
                            message.text = "Fail";
                        }
                    }
                    else
                    {
                        message.text = "Account is not available. Check your email/password and try again.";
                    }
                }
            }
        }
    }

    private string CleanInput(string input)
    {
        return input.Replace("\u200b", "").Trim();
    }

    private void Start()
    {
        if (!PhotonNetwork.IsConnected)
        {
            PhotonNetwork.ConnectUsingSettings();
        }
        else
        {
            Debug.Log("Already connected to Photon.");
            // Optionally, you can handle the case when the client is already connected.
            // For example, you might want to handle reconnection or show a message to the user.
        }
    }

    public void OnConnected()
    {
        Debug.Log("Connected to Photon");
    }

    public void OnConnectedToMaster()
    {
        Debug.Log("Connected to Master Server");
        AuthenticateWithToken(authToken);
        message.text = "Success";
    }

    public void OnDisconnected(DisconnectCause cause)
    {
        Debug.Log("Disconnected from Photon: " + cause.ToString());
        message.text = "Fail";
    }

    private void AuthenticateWithToken(string token)
    {
        string baseURL = "http://localhost:5172/User/";
        string apiURL = baseURL + token;

        // Use UnityWebRequest to make an HTTP GET request to the API with the token.
        UnityWebRequest webRequest = UnityWebRequest.Get(apiURL);

        // Send the request asynchronously and handle the response.
        webRequest.SendWebRequest().completed += operation =>
        {
            if (webRequest.result == UnityWebRequest.Result.Success)
            {
                // The request was successful. Check the status code to validate the token.
                if (webRequest.responseCode == 200)
                {
                    // Token is valid. Proceed to connect to Photon.
                    var authParameters = new System.Collections.Generic.Dictionary<string, object>
                    {
                        { "token", token }
                    };
                    PhotonNetwork.AuthValues = new AuthenticationValues(JsonUtility.ToJson(authParameters));

                    // Now, connect to Photon servers with the custom authentication values.
                    PhotonNetwork.ConnectUsingSettings();
                }
                else if (webRequest.responseCode == 400)
                {
                    // Token is invalid. Handle the error or display a message to the user.
                    Debug.Log("Invalid token. Unable to connect to Photon.");
                    message.text = "Invalid token. Unable to connect to Photon.";
                }
                else
                {
                    // Handle other response codes or errors if needed.
                    Debug.Log("Unexpected response from the API: " + webRequest.responseCode);
                    message.text = "Unexpected response from the API: " + webRequest.responseCode;
                }
            }
            else
            {
                // Handle the case when the web request fails.
                Debug.Log("Failed to connect to the authentication API: " + webRequest.error);
                message.text = "Failed to connect to the authentication API: " + webRequest.error;
            }
        };
    }

    public void OnRegionListReceived(RegionHandler regionHandler)
    {
        throw new System.NotImplementedException();
    }

    public void OnCustomAuthenticationResponse(Dictionary<string, object> data)
    {
        Debug.Log("Custom authentication response received.");
    }

    public void OnCustomAuthenticationFailed(string debugMessage)
    {
        Debug.Log("Custom authentication failed: " + debugMessage);
    }
}
